#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: Guillaume Laurent
Provides Signal-to-Noise-Ratio (SNR) for a Bruker spectrum.

Needs nmrglue.
"""

import matplotlib.pyplot as plt
import nmrglue as ng
import numpy as np
import os.path
import sys

# Customize figures
plt.rc('font', family='Times New Roman', size=16)       # default text
plt.rc('mathtext', default='regular')                   # default equations

def data_import(data_dir):
    print('\n--------------------------------------------------')
    _, data = ng.bruker.read_pdata(data_dir)            # import data
    if data.ndim != 1:
        raise NotImplementedError \
            ("SNR on", data.ndim, "dimensions is not yet supported.")
    return data

def psnr_limit(data):
    plt.ion()                                   # to avoid stop when plotting
    plt.figure()
    plt.plot(data.real)
    plt.pause(0.1)                                      # to avoid empty window
    while True:
        signal_lim = input \
            ('What are the signal limits (left right, in points) ').split()
        if (len(signal_lim) == 2) \
            and (signal_lim[0].isnumeric()) and (signal_lim[1].isnumeric()):
            signal_lim[0] = int(signal_lim[0])
            signal_lim[1] = int(signal_lim[1])
            if signal_lim[0] > signal_lim[1]:
                signal_lim = signal_lim[::-1]
            break
    plt.axvline(x=signal_lim[0], color='g', linestyle=':', linewidth=2)
    plt.axvline(x=signal_lim[1], color='g', linestyle=':', linewidth=2)
    plt.text(np.mean(signal_lim), np.max(data), 'signal', \
        color='g', ha='center')
    plt.pause(0.1)                                      # to update display
    while True:
        noise_lim = input \
            ('What are the noise limits (left right, in points)  ').split()
        if len(noise_lim) == 2 \
            and (noise_lim[0].isnumeric()) and (noise_lim[1].isnumeric()):
            noise_lim[0] = int(noise_lim[0])
            noise_lim[1] = int(noise_lim[1])
            if noise_lim[0] > noise_lim[1]:
                noise_lim = noise_lim[::-1]
            break
    plt.axvline(x=noise_lim[0], color='r', linestyle=':', linewidth=2)
    plt.axvline(x=noise_lim[1], color='r', linestyle=':', linewidth=2)
    plt.text(np.mean(noise_lim), np.max(data), 'noise', \
        color='r', ha='center')
    plt.pause(0.1)                                      # to update display
    return signal_lim, noise_lim

def signal_noise(data, signal_lim, noise_lim):
    signal = data[signal_lim[0]:signal_lim[1]].real
    noise = data[noise_lim[0]:noise_lim[1]].real
    noise_mean = np.mean(noise)
    noise_rms = np.std(noise, ddof=1)
    noise_max = (np.max(noise) - np.min(noise)) / 2
    signal_rms = np.std(signal, ddof=1)
    signal_max = np.max(signal - noise_mean)
    snnr = signal_rms / noise_rms
    snr = snnr**2 - 1
    psnr_max = signal_max / noise_max
    psnr_rms = signal_max / noise_rms
    plt.title(r'${PSNR}_{max} = $' + '{:.1f}, '.format(psnr_max) \
        + r'${PSNR}_{rms} = $' + '{:.1f}, '.format(psnr_rms) \
        + r'$SNR = $' + '{:.1f}'.format(snr))
    Lc = noise_mean + noise_rms * 1.64                  # critical limit
    Ld = noise_mean + noise_rms * 3.29                  # detection limit
    Lq = noise_mean + noise_rms * 10                    # quantification limit
    plt.axhline(y=Lc, color='tab:red', linestyle=':', linewidth=2)
    plt.axhline(y=Ld, color='tab:orange', linestyle=':', linewidth=2)
    plt.axhline(y=Lq, color='tab:green', linestyle=':', linewidth=2)
    plt.text(1.01*data.size, Lc+0.01*np.max(data), r'$L_c$', color='tab:red')
    plt.text(1.01*data.size, Ld+0.01*np.max(data), r'$L_d$', color='tab:orange')
    plt.text(1.01*data.size, Lq+0.01*np.max(data), r'$L_q$', color='tab:green')
    plt.pause(0.1)                                      # to update display
    plt.ioff()                                      # to avoid figure closing
    plt.show()                                          # to allow zooming

#%%----------------------------------------------------------------------------
### IF PROGRAM IS DIRECTLY EXECUTED
###----------------------------------------------------------------------------
if __name__ == '__main__':
    try:
        # Data directory has to be provided as an argument
        if (len(sys.argv) >= 2) and (os.path.isdir(sys.argv[1])):
            data_dir = str(sys.argv[1])
        else:
            raise ValueError \
                ('Please provide FID data directory as first argument')
        
        data = data_import(data_dir)
        signal_lim, noise_lim = psnr_limit(data)
        signal_noise(data, signal_lim, noise_lim)
        
    except ValueError as err:                           # arguments
        print('Error:', err)
        for i in range(1, len(sys.argv)):
            print('Argument', i, '=', sys.argv[i])
